/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.
 */

#include "config/configurationfile.h"

#include <fstream>
#include <iostream>

#include <logfunc.hpp>

#include "boost_assert_handler.h"
#include "config/configurationoptions.h"

using namespace std;
using boost::program_options::invalid_option_value;
using boost::program_options::options_description;
using boost::program_options::parsed_options;
using boost::program_options::parse_config_file;
using boost::program_options::variables_map;
using I2n::Logger::GlobalLogger;

//-----------------------------------------------------------------------------
// ConfigurationFile
//-----------------------------------------------------------------------------

/**
 * @brief Parameterized constructor.
 *
 * @param file_name The name and path of the file containing the configuration.
 */
ConfigurationFile::ConfigurationFile( string file_name ) :
    FileName( file_name )
{
    BOOST_ASSERT( !file_name.empty() );
}

/**
 * @brief Destructor.
 */
ConfigurationFile::~ConfigurationFile()
{
}

/**
 * @brief Reads the configuration file, storing the parsed tokens in the
 * @c variables_map.
 *
 * @param vm The variables_map object to store configuration tokens.
 *
 * @return @c true on success and @c false otherwise.
 */
bool ConfigurationFile::process( variables_map *vm )
{
    BOOST_ASSERT( !FileName.empty() );
    BOOST_ASSERT( vm != NULL );

    ifstream ifs;

    try
    {
        ifs.open( FileName.c_str() );
        if ( !ifs.good() )
        {
            GlobalLogger.error() << "Could not open " << FileName
                    << " file. Specify using argument --config-file"
                    << endl;
            return false;
        }

        ConfigurationOptions options;

        options_description config = options.get_configuration_options();
        options_description config_file_options;
        config_file_options.add( config );

        parsed_options parsed_opt = parse_config_file( ifs, config_file_options );
        store( parsed_opt, *vm );
        notify( *vm );
    }
    catch ( const invalid_option_value &e )
    {
        GlobalLogger.error() << "Invalid option value exception thrown parsing config file:"
                << e.what() << endl;
        return false;
    }
    catch ( const exception &ex )
    {
        GlobalLogger.error() << ex.what() << endl;
        return false;
    }
    catch ( ... )
    {
        GlobalLogger.error() << "Unknown exception" << endl;
        return false;
    }

    return true;
}

/**
 * @brief Store the parsed tokens in @a configuration
 *
 * @param
 * @param
 *
 * @return
 */
bool ConfigurationFile::parse(
        const variables_map &vm,
        Configuration *configuration
)
{
    BOOST_ASSERT( configuration != NULL );

    ConfigurationOptions options;

    bool configuration_parsed = options.parse_configuration_options( vm, configuration );

    return configuration_parsed;
}

